#set some parameters (default value)
pop.size = 1000     # the number of individuals in the population (1000)
mut.rate = 0.0001   # the probability that an allele mutates to the other allele per generation (0.0001)
num.loci = 100      # the number of loci (100)
num.all = 20		# the number of alleles per locus (20)
num.gens = 10000    # the number of generations to run the model, should be long enough to reach equilibrium (10000)
ploidy = 2			# the ploidy level (2)


#set initial frequencies for a locus to have only single allele present (no variation)
initial = c(1, rep(0, num.all-1))
#alternatively, uncomment the next line to set initial frequencies equal for all alleles (maximum diversity)
#initial = rep(1/num.all, num.all)

#now create an array where each row represents a locus and each column an allele at that locus
#the values are then the allele frequencies
all.freqs = rep(initial, each=num.loci)
dim(all.freqs) = c(num.loci,num.all)

#alternatively, uncomment this to set initial frequencies all equal
#all.freqs = array(1/num.all, dim=c(num.loci,num.all))

#create a vector to store some results
Hs = rep(NA, num.gens)

#now do a loop for the generations
for(g in 1:num.gens){
	#loop over loci
	for(l in 1:num.loci){
		#calculate the expected allele frequencies after mutation
		exp.freqs = (1-mut.rate)*all.freqs[l,] + rep(mut.rate/num.all,num.all)
		#now draw random number from a multinomial distribution based on expectations
		random.draw = rmultinom(1, ploidy*pop.size, exp.freqs)
		#turn those random numbers into the new allele frequencies
		new.freqs = random.draw/(ploidy*pop.size)
		all.freqs[l,]  = new.freqs
	}

	#calculate Hs
	Hs.loci = apply(all.freqs, 1, function(x) 1-sum(x^2))
	#average over loci and store
	Hs[g] = mean(Hs.loci)
	
	#every now and then print a little update
	if(g %% 1000 == 0){ print(g) }
}

#make a plot
plot(1:num.gens, Hs, ylim=c(0,1), type="l", xlab="Generation", ylab="Gene diversity", col="palevioletred2", font.lab=2)

#get average value over last 1000 generations
print(mean(Hs[(length(Hs)-1000):length(Hs)]))