#main settings of the model (default)
pop.size = 100	#the sample size of each ploidy level (100)
num.loci = 100	#the number of SNP loci (100)
ploidy1 = 2		#the ploidy level of the first half of the dataset (2)
ploidy2 = 2		#the ploidy level of the second half of the dataset (4)

allFreqs=NULL

for(l in 1:num.loci){
	#draw a random allele frequency
	freq =runif(1)
	
	#draw biallelic data for ploidy1
	ran1 = rbinom(ploidy1*pop.size,1,freq)+1
	dim(ran1) = c(pop.size, ploidy1)
	#convert to within-individual frequency data (0, 0.5 or 1 for diploid; 0, 0.25, 0.5, 0.75, or 1 for tetraploid)
	freqs1 = apply(ran1,1, function(x) sum(x==1)) / ploidy1

	#draw biallelic data for ploidy2
	ran2 = rbinom(ploidy2*pop.size,1,freq)+1
	dim(ran2) = c(pop.size, ploidy2)
	#convert to within-individual frequency data
	freqs2 = apply(ran2,1, function(x) sum(x==1)) / ploidy2
	
	#combine
	allFreqs = cbind(allFreqs, c(freqs1, freqs2))
}

#now remove dosage information
#all partial heterozygotes get freq of 0.5
allUndosFreqs = allFreqs
allUndosFreqs[allFreqs != 0 & allFreqs !=1] = 0.5

#perform a PCA on the data
pca = prcomp(allUndosFreqs)
vars = 100*pca$sdev / sum(pca$sdev)

#make a plot
ploidies = c("haploid", "diploid", "triploid", "tetraploid", "pentaploid", "hexaploid", "heptaploid", "octoploid", "nonaploid", "decaploid", "undecaploid", "dodecaploid", "tridecaploid", "tetradecaploid", "pentadecaploid", "hexadecaploid")

main = sprintf("SNPs: %s vs. %s, %d loci", ploidies[ploidy1], ploidies[ploidy2], num.loci)

plot(pca$x[,1],pca$x[,2], col=c(rep("firebrick3", pop.size), rep("dodgerblue3", pop.size)), xlab = sprintf("PCA1 (%.1f%%)", vars[1]), ylab = sprintf("PCA2 (%.1f%%)", vars[2]), main=main, font.lab=2)
legend("topright",c(ploidies[ploidy1],ploidies[ploidy2]),col=c("firebrick3","dodgerblue3"),pch=1, bg="white")



